Scriptname AngrimYngvildScr extends Quest Conditional

import Utility

SexLabFramework Property SexLab Auto
zadLibs Property DeviousLibs Auto

Actor Property PlayerRef Auto
Actor Property ArondilRef Auto
Actor Property GhostRef Auto
Quest Property AngrimYngvildQuest Auto
Quest Property AngrimApprenticeQuest Auto
Faction Property dunYngvildFaction Auto
Faction Property SexLabAnimatingFaction Auto
ObjectReference Property ChestRef Auto
Topic Property AngrimYngvildServiceTopic Auto
Topic Property AngrimYngvildTransformTopic Auto
Topic Property AngrimYngvildNaughtyTopic Auto
Topic Property AngrimYngvildCommentTopic Auto
Spell Property AngrimGhostlyStateSpell Auto
Spell Property AngrimGhostFadeSpell Auto
Spell Property AngrimLongCooldownSpell Auto
GlobalVariable Property AngrimGhostHasFed Auto
Idle Property Bound Auto
imageSpaceModifier Property VC01BlackoutIMod Auto
ObjectReference Property CenterPoint Auto

Bool Property PlayerIsCaptured Auto Conditional
Bool Property PlayerIsGhost Auto Conditional
Bool Property ArondilDead Auto Conditional
Bool Property ArondilWasHit Auto Conditional

Int ArondilCount = 0
Int MsgShown = 0
Bool IsPlugged = false

Function Setup()
	Debug.Trace( "Angrim: setting up Yngvild quest" )
	if ( PlayerRef.HasSpell( AngrimGhostlyStateSpell ) )
		PlayerIsGhost = true
	else
		PlayerIsGhost = false
		ArondilWasHit = false
	endIf
	PlayerIsCaptured = false
	ArondilDead = false
endFunction

Function Cleanup()
	Debug.Trace( "Angrim: cleaning up Yngvild quest" )
	;; Return player's faction membership to normal:
	PlayerRef.RemoveFromFaction( dunYngvildFaction )
	UnregisterForModEvent( "AnimationEnd" )
	Game.EnablePlayerControls()
endFunction

Function Maintenance()
	if ( PlayerIsGhost && ArondilDead == false )
		RegisterForModEvent( "AnimationEnd", "Reprieve" )
	endIf
	if ( isPlugged )
		RegisterForModEvent( "AnimationEnd", "StealVitality" )
	endIf
endFunction

Function Capture( Actor akActor )
	if ( PlayerIsGhost )
		;; Already a ghost, immune to the effect of ghostly touch:
		return
	endIf
	Debug.Trace( "Angrim: captured by Yngvild ghosts" )
	PlayerRef.AddToFaction( dunYngvildFaction )
	akActor.StopCombat( )
	VC01BlackoutIMod.applyCrossFade(2.0)
	Wait( 3.0 )
	PlayerRef.SheatheWeapon()
	Game.DisablePlayerControls(abMovement = true, abFighting = true, abCamSwitch = false, abLooking = false, abSneaking = false, abMenu = true, abActivate = false )
	imageSpaceModifier.removeCrossFade()
	if ( PlayerRef.IsSneaking( ) )
		;; This will cancel sneak state, if already sneaking.  Without this, speaking is broken.
		PlayerRef.StartSneaking( )
	endIf
	PlayerRef.MoveTo( CenterPoint )
	;; Put in an appropriate bound idle
	PlayerRef.PlayIdle( Bound )
	;; Steal all gear.  Devious devices and quest items should remain.
	PlayerRef.RemoveAllItems( ChestRef )
	;; Make the nearby ghost more effective in a fight:
	GhostRef.ModActorValue( "Health", 200 )
endFunction

Function MakeGhost()
	;; Set the flag early, to cut off blocking dialogue that depends on this.
	PlayerIsGhost = true
	VC01BlackoutIMod.applyCrossFade(2.0)
	Wait( 3.0 )
	AngrimLongCooldownSpell.Cast( ArondilRef, ArondilRef )
	Game.ForceThirdPerson()
	PlayerRef.AddSpell( AngrimGhostlyStateSpell, false )
	PlayerRef.AddSpell( AngrimGhostFadeSpell, false )
	Wait( 7.0 )
	AngrimApprenticeQuest.SetStage( 100 )
	imageSpaceModifier.removeCrossFade()
	RegisterForModEvent( "AnimationEnd", "Reprieve" )
	RegisterForSingleUpdate( 75.0 )
	Game.DisablePlayerControls(abMovement = false, abFighting = true, abCamSwitch = false, abLooking = false, abSneaking = false, abMenu = true, abActivate = true )
	ArondilRef.Say( AngrimYngvildTransformTopic )
	RemovePlug()
endFunction

Event Reprieve( string eventName, string argString, float argNum, form sender )
	if ( ArondilDead )
		return
	endIf
	if ( MsgShown < 6 )
		Debug.Notification( "You feel some strength returning, as your captor is distracted." )
	endIf
	Game.EnablePlayerControls()
	float fDelay = 4.0 + ( 2.0 * ArondilCount )
	RegisterForSingleUpdate( fDelay )
endEvent

Event OnUpdate()
	if ( ArondilDead )
		return
	endIf
	bool ArondilGemStolen = ( ArondilRef.IsInFaction( dunYngvildFaction ) != 1 )
	if ( AngrimGhostHasFed.Value > 0 )
		Debug.Trace( "Angrim: Yngvild OnUpdate event for fading strength" )
		if ( ArondilGemStolen )
			Debug.Notification( "You feel a surge of returning strength." )
			Game.EnablePlayerControls()
			return
		endIf
		if ( MsgShown < 6 )
			Debug.Notification( "Your strength rapidly fades..." )
		endIf
		AngrimGhostHasFed.Value = 0
		Game.DisablePlayerControls(abMovement = false, abFighting = true, abCamSwitch = false, abLooking = false, abSneaking = false, abMenu = true, abActivate = true )
		if ( ArondilWasHit )
			;; Arondil was hit by the player
			ArondilWasHit = false
			GhostRef.StopCombat()
			ArondilRef.StopCombat()
			ArondilRef.Say( AngrimYngvildNaughtyTopic )
		endIf
		RegisterForSingleUpdate( 100.0 )
	else
		Debug.Trace( "Angrim: Yngvild OnUpdate event for a bout with Arondil" )
		if ( PlayerRef.IsInFaction( SexLabAnimatingFaction ) )
			;; Already animating, probably from some other mod.  Just skip this round.
			RegisterForSingleUpdate( 100.0 )
			return
		endIf
		if ( ArondilGemStolen )
			Debug.Notification( "You feel a surge of returning strength." )
			Game.EnablePlayerControls()
			return
		endIf
		if ( PlayerRef.GetDistance( ArondilRef ) > 150 )
			if ( MsgShown < 6 )
				Debug.Notification( "You are drawn to your captor's side..." )
			endIf
			VC01BlackoutIMod.applyCrossFade(2.0)
			Wait( 3.0 )
			imageSpaceModifier.removeCrossFade()
			PlayerRef.MoveTo( CenterPoint )
		endIf
		if ( ArondilWasHit )
			;; Arondil was hit by with the player:
			ArondilWasHit = false
			GhostRef.StopCombat()
			ArondilRef.StopCombat()
			ArondilRef.Say( AngrimYngvildNaughtyTopic )
		else
			ArondilRef.Say( AngrimYngvildServiceTopic )
		endIf
		;; Failsafe delay, in case something preempts the usual dialogue topics:
		RegisterForSingleUpdate( 200.0 )
	endIf
	MsgShown += 1
endEvent

Function WasHit()
	Debug.Trace( "Angrim: Arondil was hit by the player" )
	ArondilWasHit = true
endFunction

Function Dying()
	;; If Arondil dies by indirect means, make sure we return control:
	Debug.Trace( "Angrim: Arondil was killed" )
	ArondilDead = true
	Game.EnablePlayerControls()
endFunction

Function AddPlug()
	;; We are not really using Devious plugs for this anymore; too many potential conflicts with other gear.
	isPlugged = true
	PlayerIsCaptured = true
	RegisterForModEvent( "AnimationEnd", "StealVitality" )
	SoloAct()
endFunction

Function RemovePlug()
	isPlugged = false
endFunction

Event StealVitality( string eventName, string argString, float argNum, form sender )
	UnregisterForModEvent( "AnimationEnd" )
	MakeGhost()
endEvent

Function SoloAct( )
	Actor[] actors = new Actor[1]
	Actor victim = None
	sslBaseAnimation[] anims
	sslBaseAnimation preferred
	actors[0] = PlayerRef

	Debug.Trace( "Angrim: starting solo animation" )

	;; Use the Dwemer Machine (from Estrus) if it is available.  Otherwise, pick any solo anim:
	preferred = SexLab.GetAnimationByName( "Dwemer Machine" )
	if ( preferred )
		anims = new sslBaseAnimation[1]
		anims[0] = preferred
	else
		anims = SexLab.GetAnimationsByTags( 1, Tags="Solo", TagSuppress="Pillory" )
	endIf

	SexLab.StartSex(actors, anims, victim, centerOn=PlayerRef, allowBed=false)
endFunction

Function DuoAct( Actor akActor, Bool abRough=true )
	Actor[] actors = new Actor[2]
	Actor victim = None
	sslBaseAnimation[] anims
	actors[0] = PlayerRef
	actors[1] = akActor

	Debug.Trace( "Angrim: starting pair animation" )
	AngrimLongCooldownSpell.Cast( akActor, akActor )

	if ( akActor == ArondilRef )
		ArondilCount += 1
	endIf
	if ( abRough )
		victim = PlayerRef
	endIf

	;; Look for bound / aggressive animations:
	anims = SexLab.GetAnimationsByTags( 2, Tags="Binding,Rough,Aggressive", TagSuppress="Pillory", RequireAll=false )

	SexLab.StartSex(actors, anims, victim, centerOn=PlayerRef, allowBed=false)
endFunction

Function Spanking( Actor akActor )
	Actor[] actors = new Actor[2]
	Actor victim = playerRef
	sslBaseAnimation[] anims
	actors[0] = PlayerRef
	actors[1] = akActor

	Debug.Trace( "Angrim: starting spanking animation" )
	AngrimLongCooldownSpell.Cast( akActor, akActor )

	anims = new sslBaseAnimation[1]
	anims[0] = SexLab.GetAnimationByName( "Zaz OTK Spanking" )

	if ( anims[0] )
		SexLab.StartSex(actors, anims, victim, centerOn=PlayerRef, allowBed=false )
	endIf
endFunction
